#include "includes.h"
#include <stdio.h>

long dchart[5] = {50000, 30000, 15000, 10000, 5000};
char nchart[5][4] = {"AAA", "BBB", "CCC", "DDD", "EEE"};

HISCORES::HISCORES()
{
    for(int i = 0; i < 5; i++)
    {
        score[i] = dchart[i];
        for(int j = 0; j < 4; j++)
        {
            name[i][j] = nchart[i][j];
        }
    }
}

int HISCORES::load(char * fn)
{
    PACKFILE * fp;
#ifdef __unix__
    char *get_home_dir(void);
    char buf[512];
    char *home = get_home_dir();

    snprintf(buf, sizeof(buf), "%s/.smashteroid/%s", home? home:".", fn);
    fn = buf;
#endif

    if(!exists(fn))
    {
        return 0;
    }
    fp = pack_fopen(fn, "r");
    if(fp == NULL)
    {
        return 0;
    }

    for(int i = 0; i < 5; i++)
    {
        score[i] = pack_igetl(fp);
        for(int j = 0; j < 4; j++)
        {
            name[i][j] = pack_getc(fp);
        }
    }
    pack_fclose(fp);

    return 1;
}

int HISCORES::save(char * fn)
{
    PACKFILE * fp;
#ifdef __unix__
    char *get_home_dir(void);
    char buf[512];
    char *home = get_home_dir();

    snprintf(buf, sizeof(buf), "%s/.smashteroid/%s", home? home:".", fn);
    fn = buf;
#endif

    fp = pack_fopen(fn, "w");
    if(fp == NULL)
    {
        return 0;
    }

    for(int i = 0; i < 5; i++)
    {
        pack_iputl(score[i], fp);
        for(int j = 0; j < 4; j++)
        {
            pack_putc(name[i][j], fp);
        }
    }
    pack_fclose(fp);

    return 1;
}

int HISCORES::place(long iscore)
{
    for(int i = 0; i < 5; i++)
    {
        if(iscore >= score[i])
        {
            return i;
        }
    }
    return -1;
}

void HISCORES::make_room(int spot)
{
    for(int i = 4; i > spot; i--)
    {
        score[i] = score[i - 1];
        for(int j = 0; j < 4; j++)
        {
            name[i][j] = name[i - 1][j];
        }
    }
}

long HISCORES::get_score(int spot)
{
    return score[spot];
}

char * HISCORES::get_name(int spot)
{
    return name[spot];
}

void HISCORES::put_score(long iscore, int spot)
{
    score[spot] = iscore;
}

void HISCORES::put_name(char * iname, int spot)
{
    for(int i = 0; i < 4; i++)
    {
        name[spot][i] = iname[i];
    }
}
