#include "includes.h"

MY_MENU_ITEM::MY_MENU_ITEM()
{
    for(int i = 0; i < 128; i++)
    {
        text[i] = '\0';
    }
    left_proc = NULL;
    right_proc = NULL;
    enter_proc = NULL;
}

void MY_MENU_ITEM::initialize(char * itext, int ix, int iy, int ox, int oy, void(*lp)(), void(*rp)(), void(*ep)())
{
    for(int i = 0; i < 128 && i <= strlen(itext); i++)
    {
        text[i] = itext[i];
    }
    x = ix;
    y = iy;
    is_ox = ox;
    is_oy = oy;
    left_proc = lp;
    right_proc = rp;
    enter_proc = ep;
}

void MY_MENU_ITEM::rename(char * itext)
{
    strcpy(text, itext);
}

int MY_MENU_ITEM::get_x(void)
{
    return x;
}

int MY_MENU_ITEM::get_y(void)
{
    return y;
}

int MY_MENU_ITEM::get_ox(void)
{
    return is_ox;
}

int MY_MENU_ITEM::get_oy(void)
{
    return is_oy;
}

void MY_MENU_ITEM::do_left_proc(void)
{
    if(left_proc != NULL)
    {
        left_proc();
        t3ss_play_sound_reserved(land_sound, 0, I_NULL, I_NULL);
    }
}

void MY_MENU_ITEM::do_right_proc(void)
{
    if(right_proc != NULL)
    {
        right_proc();
        t3ss_play_sound_reserved(land_sound, 0, I_NULL, I_NULL);
    }
}

void MY_MENU_ITEM::do_enter_proc(void)
{
    if(enter_proc != NULL)
    {
        t3ss_play_sound_reserved(explode_sound, 0, I_NULL, I_NULL);
        enter_proc();
    }
}

void MY_MENU_ITEM::draw(BITMAP * bp, NCDFONT * fp)
{
    ncd_printf(bp, fp, x, y, text);
}

MY_MENU::MY_MENU()
{
    item_count = 0;
    active = 0;
    selected_item = 0;
    update_proc = NULL;
    sel_anim = NULL;
    sel_anim_frame = 0;
    menu_font = NULL;
}

void MY_MENU::initialize(void(*up)(), NCDFONT * fp, ANIMATION * ap)
{
    update_proc = up;
    sel_anim = ap;
    menu_font = fp;
}

void MY_MENU::add_item(char * itext, int ix, int iy, int ox, int oy, void(*lp)(), void(*rp)(), void(*ep)())
{
    if(item_count < MAX_MENU_ITEMS)
    {
        item[item_count].initialize(itext, ix, iy, ox, oy, lp, rp, ep);
        item_count++;
    }
}

void MY_MENU::rename_item(int inum, char * itext)
{
    item[inum].rename(itext);
}

void MY_MENU::set_parent(int ip)
{
    parent = ip;
}

int MY_MENU::get_parent(void)
{
    return parent;
}

void MY_MENU::select_item(int inum)
{
    selected_item = inum;
}

void MY_MENU::clear(void)
{
    item_count = 0;
}

void MY_MENU::activate(void)
{
    active = 1;
}

void MY_MENU::deactivate(void)
{
    active = 0;
}

void MY_MENU::do_controls(void)
{
    if(active)
    {
        ncd_joy_poll();
        if(key[KEY_UP] || ncd_joy_up(0))
        {
            t3ss_play_sound_reserved(land_sound, 0, I_NULL, I_NULL);
            selected_item--;
            if(selected_item < 0)
            {
                selected_item = item_count - 1;
            }
            while(key[KEY_UP] || ncd_joy_up(0))
            {
                ncd_joy_poll();
                if(update_proc != NULL)
                {
                    update_proc();
                }
            }
        }
        if(key[KEY_DOWN] || ncd_joy_down(0))
        {
            t3ss_play_sound_reserved(land_sound, 0, I_NULL, I_NULL);
            selected_item++;
            if(selected_item >= item_count)
            {
                selected_item = 0;
            }
            while(key[KEY_DOWN] || ncd_joy_down(0))
            {
                ncd_joy_poll();
                if(update_proc != NULL)
                {
                    update_proc();
                }
            }
        }
        if(key[KEY_LEFT] || ncd_joy_left(0))
        {
            item[selected_item].do_left_proc();
            while(key[KEY_LEFT] || ncd_joy_left(0))
            {
                ncd_joy_poll();
                if(update_proc != NULL)
                {
                    update_proc();
                }
            }
        }
        if(key[KEY_RIGHT] || ncd_joy_right(0))
        {
            item[selected_item].do_right_proc();
            while(key[KEY_RIGHT] || ncd_joy_right(0))
            {
                ncd_joy_poll();
                if(update_proc != NULL)
                {
                    update_proc();
                }
            }
        }
        if(key[KEY_ENTER] || ncd_joy_button(0, 0) || ncd_joy_button(0, 1))
        {
            item[selected_item].do_enter_proc();
            while(key[KEY_ENTER] || ncd_joy_button(0, 0) || ncd_joy_button(0, 1))
            {
                ncd_joy_poll();
                if(update_proc != NULL)
                {
                    update_proc();
                }
            }
        }
    }
}

void MY_MENU::draw(BITMAP * bp)
{
    if(active)
    {
        for(int i = 0; i < item_count; i++)
        {
            if(menu_font != NULL)
            {
                item[i].draw(bp, menu_font);
            }
        }
        if(sel_anim != NULL)
        {
            draw_animation(sel_anim, sel_anim_frame, bp, item[selected_item].get_x() + item[selected_item].get_ox(), item[selected_item].get_y() + item[selected_item].get_oy());
            sel_anim_frame++;
        }
    }
}
