#ifndef OBJECTS_H
#define OBJECTS_H

#include "animate.h"

#define BOUNDARY_STOP       0
#define BOUNDARY_DEACTIVATE 1
#define BOUNDARY_REVERSE    2

class BASIC_SPRITE
{

public:

    BASIC_SPRITE();

    /* initialize the sprite */
    void initialize(ANIMATION * anip);
    void set_friction(fix ifx, fix ify);
    void set_acceleration(fix iax, fix iay);
    void set_max_speed(fix imvx, fix imvy);
    void set_collision(int ix, int iy, int iw, int ih);
    void set_flash_color(int col);
    void set_boundary_top(fix t, int type);
    void set_boundary_bottom(fix b, int type);
    void set_boundary_left(fix l, int type);
    void set_boundary_right(fix r, int type);
    void set_boundary(fix l, fix t, fix r, fix b, int type);
    void set_boundary_at_edge(BITMAP * bp, int type);
    void set_boundary_past_edge(BITMAP * bp, int type);
    void set_life(int l);

    /* set position and speed */
    void set_position(fix ix, fix iy);
    void set_speed(fix ivx, fix ivy);
    void set_xspeed(fix ivx);
    void set_yspeed(fix ivy);
    void reverse_x(void);
    void reverse_y(void);
    fix get_x(void);
    fix get_y(void);
    fix get_ox(void);
    fix get_oy(void);
    fix get_xspeed(void);
    fix get_yspeed(void);
    void accelerate(int dx, int dy);
    void set_animation(int dir);
    void activate(void);
    void deactivate(void);
    int active(void);
    void update_position(void);

    void spawn(BASIC_SPRITE * sp2, fix ox, fix oy, fix vx, fix vy);
    void spawn_center(BASIC_SPRITE * sp2, fix ox, fix oy, fix vx, fix vy);
    void spawn_relative(BASIC_SPRITE * sp2, fix ox, fix oy, fix ivx, fix ivy);
    void spawn_center_relative(BASIC_SPRITE * sp2, fix ox, fix oy, fix ivx, fix ivy);
    void follow(BASIC_SPRITE * sp2, double speed);
    void follow_center(BASIC_SPRITE * sp2, double speed);
        
    /* check collision of 2 sprites */
    int collide(BASIC_SPRITE * sp2);
    int collide_perfect(BASIC_SPRITE * sp2);

    /* check collision between sprite and rectangle */
    int collide_rect(fix x, fix y, fix width, fix height);

    void flash(void);

    /* draw the sprite */
    void draw(BITMAP * bp);

private:

    /* location */
    fix x, y;

    /* velocities */
    fix vx, vy;
    fix mvx, mvy;
    fix minx, miny;
    fix maxx, maxy;
    int btypel, btyper, btypet, btypeb;

    /* friction and acceleration */
    fix fx, fy;
    fix ax, ay;

    /* active status */
    int is_active;

    /* pointer to animation the sprite uses */
    ANIMATION * ap;

    /* current animation frame */
    unsigned long frame;
    int fdir;
    int movingx;
    int movingy;

    /* collision stuff */
    int cx, cy, cw, ch;
    int flashing, flash_color;
    int life;

};

#endif
